bl_info = {
	"name": "Msp GL datafiles",
	"author": "Mikko Rasa",
	"location": "File > Export",
	"description": "Export Msp GL data",
	"category": "Import-Export" }

if "bpy" in locals():
	import imp
	for sub in "armature", "export_armature", "export_mesh", "export_object", "export_scene", "mesh", "outfile", "properties", "util":
		if sub in locals():
			imp.reload(locals()[sub])

import bpy
from bpy_extras.io_utils import ExportHelper

class ExportMspGLBase(ExportHelper):
	show_progress = bpy.props.BoolProperty(name="Show progress", description="Display progress indicator while exporting", default=True)

	def execute(self, context):
		exporter = self.create_exporter()
		self.prepare_exporter(exporter)
		exporter.export(context, self.filepath)
		return {"FINISHED"}

	def create_exporter(self):
		raise Exception("create_exporter must be overridden")

	def prepare_exporter(self, exporter):
		for k, v in self.as_keywords().items():
			setattr(exporter, k, v)

class ExportMspGLMeshBase(ExportMspGLBase):
	use_strips = bpy.props.BoolProperty(name="Use strips", description="Combine faces into triangle strips", default=True)
	use_degen_tris = bpy.props.BoolProperty(name="Use degen tris", description="Concatenate triangle strips with degenerate triangles", default=False)

	def draw(self, context):
		self.general_col = self.layout.column()

		col = self.layout.column()
		col.label("Triangle strips")
		col.prop(self, "use_strips")
		col.prop(self, "use_degen_tris")

class ExportMspGLMesh(bpy.types.Operator, ExportMspGLMeshBase):
	bl_idname = "export_mesh.mspgl_mesh"
	bl_label = "Export Msp GL mesh"

	filename_ext = ".mesh"

	def create_exporter(self):
		from .export_mesh import MeshExporter
		return MeshExporter()

class ExportMspGLObject(bpy.types.Operator, ExportMspGLMeshBase):
	bl_idname = "export_mesh.mspgl_object"
	bl_label = "Export Msp GL object"

	filename_ext = ".object"

	textures = bpy.props.EnumProperty(name="Textures", description="Export textures", default="REF",
		items=(("NONE", "None", "Ignore textures"),
			("REF", "Referenced", "Reference external data"),
			("INLINE", "Inline", "Embed textures in the object")))

	separate_mesh = bpy.props.BoolProperty(name="Separate mesh", description="Write mesh data into a separate file", default=False)
	separate_tech = bpy.props.BoolProperty(name="Separate technique", description="Write technique data into a separate file", default=False)
	shared_resources = bpy.props.BoolProperty(name="Shared resources", description="Use global names for resource files to enable sharing", default=True)

	export_lods = bpy.props.BoolProperty(name="Export LoDs", description="Export all levels of detail", default=True)

	def create_exporter(self):
		from .export_object import ObjectExporter
		return ObjectExporter()

	def draw(self, context):
		super().draw(context)

		col = self.general_col
		col.prop(self, "export_lods")
		col.prop(self, "textures")
		col.separator()

		self.layout.separator()

		col = self.layout.column()
		col.label("Files")
		col.prop(self, "separate_mesh")
		col.prop(self, "separate_tech")
		if self.separate_mesh or self.separate_tech:
			col.prop(self, "shared_resources")

class ExportMspGLArmature(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export.mspgl_armature"
	bl_label = "Export Msp GL armature"

	filename_ext = ".arma"

	def create_exporter(self):
		from .export_armature import ArmatureExporter
		return ArmatureExporter()

class ExportMspGLScene(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export_scene.mspgl_scene"
	bl_label = "Export Msp GL scene"

	filename_ext = ".scene"

	resource_collection = bpy.props.BoolProperty(name="Resource collection", description="Put resources to a single collection file", default=True)

	def create_exporter(self):
		from .export_scene import SceneExporter
		return SceneExporter()

	def draw(self, context):
		col = self.layout.column()
		col.prop(self, "resource_collection")

def menu_func_export(self, context):
	self.layout.operator(ExportMspGLMesh.bl_idname, text="Msp GL mesh")
	self.layout.operator(ExportMspGLObject.bl_idname, text="Msp GL object")
	self.layout.operator(ExportMspGLArmature.bl_idname, text="Msp GL armature")
	self.layout.operator(ExportMspGLScene.bl_idname, text="Msp GL scene")

from .properties import MspGLMeshProperties, MspGLObjectProperties

def register():
	bpy.utils.register_module(__name__)

	bpy.types.INFO_MT_file_export.append(menu_func_export)

	from .properties import register_properties
	register_properties()

def unregister():
	bpy.utils.unregister_module(__name__)

	bpy.types.INFO_MT_file_export.remove(menu_func_export)

if __name__=="__main__":
	register()
